@@ -3,9 +3,9 @@ from time import sleep, monotonic
 from math import ceil, inf
 from os import environ
 
-from psutil import Process, cpu_count, cpu_times
+from psutil import cpu_count, cpu_times
 
-from .cgroups import get_cgroups_cpu_quota
+from .cgroups import get_cgroups_cpu_quota, get_cpu_used
 
 # How often to poll the CPU, compared to the moving-average of the job-start-interval
 CPU_IDLE_POLL_DIVISOR = 20
@@ -116,16 +116,23 @@ def wait_for_idle_cpus(
 ):
     """CPU-gated wait_for_slot implementation
 
-    Creates a wait_for_slot-callable that will let through at least one concurrent job, and additional jobs as
-    long as `required` cpu:s are idle. It is cgroup-aware, and will not allow current process and subprocesses to
-    consume more CPU than configured cgroup-limit.
+    Creates a wait_for_slot-callable that will let through at least one concurrent job, and additional jobs
+    as long as `required` cpu:s are idle. It is cgroup-aware, and will not fetch new jobs that would break
+    configured cgroup-limit. It will also measure the total CPU _of the current cgroup_, as reported by
+    /sys/fs/cgroup/cpu.stat. Typically, when running under Linux, in a container, or in the host operating
+    system, this will do what's expected.
+
+    This gater will accept new jobs as long as the current cgroup (container) aren't using
+    `POTHEAD_CPU_PROMISED` cores, or if further idle CPU seems to be available.
 
     When a worker is spawned, but before it has received its job request, it will have the CPU amount
     specified in the `required` parameter artificially reserved for it. The `keep_cpu_reservation`
     parameter can be used for jobs that when started don't immediately use the full amount of CPU that
-    they will do eventually. In order to avoid spawning lots of such jobs in a short time period before
-    CPU usage has settled, the artifical CPU reservation can be kept by setting `keep_cpu_reservation`
-    to True. When doing this, the job should at some later point during its request processing reset the
+    they will do eventually.
+
+    In order to avoid spawning lots of such jobs in a short time period before CPU usage has settled, the
+    artifical CPU reservation can be kept by setting `keep_cpu_reservation` to True. When doing this, the job
+     should at some later point during its request processing reset the
     CPU reservation by calling the function stored under the "RESET_CPU_RESERVATION" key in the WSGI
     request environment, like so:
 
@@ -137,12 +144,6 @@ def wait_for_idle_cpus(
     worker_list = WorkerList()
     cpu_wait_time = MovingAverage(1, 0.7)
 
-    process = Process()
-
-    def cpu_used_in_process():
-        times = process.cpu_times()
-        return times.user + times.system + times.children_user + times.children_system
-
     cpu_quota = get_cgroups_cpu_quota()
     if cpu_quota < inf:
         if not max_concurrent:
@@ -174,7 +175,7 @@ def wait_for_idle_cpus(
         start_time = monotonic()
 
         idle = InertialTimeDerivate(lambda: cpu_times().idle, 0, inertia)
-        cpu_used = InertialTimeDerivate(cpu_used_in_process, cpu_count(), inertia)
+        cpu_used = InertialTimeDerivate(get_cpu_used, cpu_count(), inertia)
 
         cpu_poll_interval = max(
             CPU_IDLE_POLL_MIN_INTERVAL,
