@@ -8,6 +8,7 @@ the inner WSGI app.
 
 import io
 import multiprocessing
+import psutil
 from logging import Logger
 from multiprocessing.connection import Connection
 from tblib import pickling_support
@@ -202,8 +203,7 @@ class SubprocessMiddleware:
                         False
                     ), f"Got unknown message type from WSGI subprocess: {msg}"
         finally:
-            subprocess.terminate()
-            subprocess.join()
+            _kill_process_and_its_descendants(subprocess)
             request_sender_thread.join()
 
     @staticmethod
@@ -290,3 +290,44 @@ def subprocess_main(
         except Exception as exc:
             msg_writer.send(MsgExceptionRaised(exc))
             raise
+
+
+def _suspend_recursive(pid: psutil.Process) -> Iterable[psutil.Process]:
+    """Gets a list of process and descendants. The only way to do it ~"atomically", is to suspend
+    them in the process, preventing them from forking or dying while we're looking the other way.
+
+    Returns a list of now suspended processes, including the root specified by pid"""
+    try:
+        proc = psutil.Process(pid)
+        proc.suspend()
+    except psutil.NoSuchProcess:
+        return []
+
+    descendants = {proc}
+    while True:
+        prior_count = len(descendants)
+        for child in proc.children(recursive=True):
+            if child not in descendants:
+                try:
+                    child.suspend()
+                except psutil.NoSuchProcess:
+                    continue
+                descendants.add(child)
+        if len(descendants) == prior_count:
+            break
+
+    return descendants
+
+
+def _kill_process_and_its_descendants(process: multiprocessing.Process):
+    descendants = _suspend_recursive(process.pid)
+
+    for descendant in descendants:
+        try:
+            descendant.kill()
+        except psutil.NoSuchProcess:
+            # Without a defined order, descendants might be already orphaned, adopted and killed
+            # by init when we get to them
+            continue
+
+    process.join()
