@@ -1,5 +1,6 @@
 import io
 import os
+import subprocess
 from logging import getLogger
 from psutil import Process
 from threading import Thread
@@ -124,6 +125,9 @@ def wrapped_app(environ: Environ, start_response: StartResponse) -> Iterable[byt
             t = Thread(target=sleep_forever, daemon=False)
             t.start()
 
+            # This subprocess represents something that the app might spawn, that doesn't automatically exit on its own
+            subprocess.Popen(["bash", "-c", "while true; do sleep 1000; echo HI FROM APP; done"])
+
             if request.path == "/test_successful_subprocess_is_shut_down":
                 start_response("200 OK", [], None)
             else:
@@ -302,7 +306,13 @@ class SubprocessMiddlewareTest(TestCase):
         middleware = SubprocessMiddleware(LOGGER, wrapped_app)
 
         proc = Process(os.getpid())
+
+        init_process = proc
+        while init_process.pid != 1:
+            init_process = init_process.parent()
+
         subprocesses_before = set(proc.children(recursive=True))
+        init_subprocesses_before = set(init_process.children(recursive=True))
 
         try:
             request_func(middleware)
@@ -319,4 +329,22 @@ class SubprocessMiddlewareTest(TestCase):
                 for p in extra_processes:
                     p.terminate()
 
-                assert False, "Request subprocess didn't exit"
+                assert False, f"Request subprocess didn't exit: {extra_processes}"
+
+            # If the app leaked running subprocesses upon being killed, these will be orphaned by
+            # the init process. To avoid getting false positives on this (in case the init process has
+            # adopted some unrelated subprocesses while the test was being run), we only conclude that
+            # a process was leaked if its cmdline matches a known string.
+            init_subprocesses_after = set(init_process.children(recursive=True))
+            if init_subprocesses_before != init_subprocesses_after:
+                # Avoid hanging the test itself in case it fails
+                extra_processes = init_subprocesses_after.difference(init_subprocesses_before)
+                leaked_subprocess = None
+                for p in extra_processes:
+                    if any(word for word in p.cmdline() if "HI FROM APP" in word):
+                        leaked_subprocess = p
+                        print(f"Leaked subprocess: {leaked_subprocess} ({leaked_subprocess.cmdline()})")
+                    p.terminate()
+
+                if leaked_subprocess:
+                    assert False, f"Subprocess was leaked: {leaked_subprocess}"
