@@ -216,7 +216,9 @@ class Server:
         del app.wait_for_slot
         self.app = app
         self.app = (
-            SubprocessMiddleware(self.app) if use_request_subprocess else self.app
+            SubprocessMiddleware(LOGGER, self.app)
+            if use_request_subprocess
+            else self.app
         )
         self.app = (
             OutOfBandResponder(
